<?php
namespace Teknisa\COZ\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Error;
use Planejamento\Logic\PlanejamentoProducao\DistProducaoLogic;
use Teknisa\COZ\Services\Coz07200DistribuicaoDaRefeicaoService;

class Coz07200DistribuicaoDaRefeicao extends \Teknisa\GER\Controllers\TeknisaCrud {

	protected $dataSourceName = 'distribuicao_refeicao';
	protected $parameterBagColumns = array('NRORG', 'CDCENTRAL', 'NRDISTPROD', 'DTPRODUCAO');
	protected $distribuicaoRefeicaoService;
	protected $novoCodigo;
	protected $entityManager;
	protected $util;
	protected $connection;

	public function __construct($crudService, $parameterBag, $dataSourceManager, $environment, Coz07200DistribuicaoDaRefeicaoService $distribuicaoRefeicaoService, $entityManager, $novoCodigo, $util, $connection){
		$this->crudService                  = $crudService;
		$this->parameterBag                 = $parameterBag;
		$this->dataSourceManager            = $dataSourceManager;
		$this->environment                  = $environment;
		$this->distribuicaoRefeicaoService  = $distribuicaoRefeicaoService;
		$this->entityManager				= $entityManager;
		$this->novoCodigo					= $novoCodigo;
		$this->util 						= $util;
		$this->connection 					= $connection;
	}


	public function procedureDistribuiProducao(DTO\Request $request, DTO\Response $response){
			$parametros = $request->getRow();

			$nrdistribuicao = $this->distribuicaoRefeicaoService->DistribuiProducao(
				$parametros['OPERACAO'], $parametros['CDCENTRAL'], $parametros['NRDISTPROD'],
				$parametros['DTINI'], $parametros['DTFIN'], $parametros['NRLISTA_SERV'], $parametros['NRORG'],
				$parametros['CDOPERADOR'], $parametros['CONSUMO']);

			$response->addDataSet(new DataSet('NRDISTPROD', explode(';', $nrdistribuicao['P_RETORNO'])));
	}
	
	public function procedureEfetuaDistribuicao(DTO\Request $request, DTO\Response $response){
		try {
			$parametros = $request->getRow();
			$nrcontrole = '';
			if ($parametros['OPERACAO'] == "G") {
				$nrcontrole = $this->distribuicaoRefeicaoService->ControleNovoCodigo($parametros['NRDISTPROD'], $parametros['CDCENTRAL'], $parametros['NRORG']);
				$nrcontrole = $nrcontrole['P_NRCONTROLE'];
			}
		/*	var_dump($parametros);
			var_dump($nrcontrole);die;*/
			
			if($nrcontrole != 'S'){
				$this->distribuicaoRefeicaoService->iniciaConexao();
				$efetivaDist = $this->distribuicaoRefeicaoService->EfetivaDistribuicao(
					$parametros['NRDISTPROD'],
					$parametros['OPERACAO'], 
					$parametros['CDCENTRAL'],
					$parametros['CDOPERADOR'],
					$parametros['DATAPROC'],
					$parametros['DATARETIRADA'],
					$parametros['DATACARDAPIO'],
					$parametros['NRORG'],
					$parametros['NRLISTAPROD'],
					$nrcontrole
				);
				
				if ($parametros['OPERACAO'] == "G") {
					$this->distribuicaoRefeicaoService->deletaCodigoControle($parametros['NRORG'], $nrcontrole);
				}
				$this->distribuicaoRefeicaoService->realizaCommit();
				$this->util->deleteLista($parametros['NRLISTAPROD']);
				
				$response->addDataSet(new DataSet ('EFETIVADIST', $efetivaDist));
			} else{
				$response->addDataSet(new DataSet ('EFETIVADIST', array('P_RETORNO' => $nrcontrole)));
			}
		} catch (\Exception $e) {
			$this->distribuicaoRefeicaoService->realizaRollBack();
			$this->distribuicaoRefeicaoService->atualizaDitribuicaoRollback($parametros['NRDISTPROD'], $parametros['NRORG']);
			if ($parametros['OPERACAO'] == "G" && $nrcontrole) {
				$this->distribuicaoRefeicaoService->deletaCodigoControle($parametros['NRORG'], $nrcontrole);
			}
			$response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
		}
	}

	public function atualizaDistribuicaoRefeicao(DTO\Request $request, DTO\Response $response){
		try {
			$parametros = $request->getRow();
			$nrorg = $parametros['NRORG'];
			$nrDistCard = $parametros['NRDISTCARD'];
			$nrDistProd = $parametros['NRDISTPROD'];
			$filial = $parametros['CDFILIAL'];
			$servico = $parametros['CDSERVFILI'];
			$cliente = $parametros['CDCLIENTE'];
			$destino = $parametros['CDDESTFILI'];
			$nrembdistcard = $parametros['NREMBDISTCARD'];
			$observacao = $parametros['DSOBSETIQUETA'];
			$idPrincipal = $parametros['IDPRINCIPAL'];
			$seqPrato = $parametros['NRSEQPRATCAR'];
			$tipoProduto = $parametros['IDTIPOPROD'];
			
			$this->distribuicaoRefeicaoService->iniciaConexao();
			
			if ($tipoProduto == 'NENHUM') {
				$this->trataPratoPrincipal($nrorg, $nrDistCard, $idPrincipal, $nrembdistcard);
				$this->distribuicaoRefeicaoService->atualizaDistribuicaoPrato(
					$nrorg, $nrDistCard, $nrembdistcard, $observacao,
					$idPrincipal, $seqPrato
				);
			} else {
				$this->trataProdutoPrincipal($nrorg, $nrDistProd, $filial, $servico, $cliente, $destino, $idPrincipal, $nrembdistcard, $nrDistCard);
				$this->distribuicaoRefeicaoService->atualizaDistribuicaoProduto(
					$nrorg, $nrDistProd, $nrembdistcard,
					$observacao, $idPrincipal, $nrDistCard
				);
			}
			
			$this->distribuicaoRefeicaoService->realizaCommit();

		} catch(\Exception $e) {
			$this->distribuicaoRefeicaoService->realizaRollBack();
			$response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
		}
	}

	public function atualizaQuantidade (DTO\Request $request, DTO\Response $response){
		try {
			$rows = $request->getRow();		
			foreach ($rows as $parametros) {
                $idtipoprod = $parametros['IDTIPOPROD'];
                $nrorg = $parametros['NRORG'];
                $nrdistcard = $parametros['NRDISTCARD'];
                $cdproduto = $parametros['CDPRODUTO'];
                $qttotproduto = round($parametros['QTPRODUTO'],5);
                if ($idtipoprod == 'NENHUM') {
                    $this->distribuicaoRefeicaoService->atualizaQuantidadePrato(
                        $nrorg, $nrdistcard, $cdproduto, $qttotproduto
                    );
                } else {
                    $cdfilial = $parametros['CDFILIAL'];
                    $nrdistprod = $parametros['NRDISTPROD'];
                    $cdservfili = $parametros['CDSERVFILI'];
                    $dtcardapio = $parametros['DTCARDAPIO'];
                    $cdcliente = $parametros['CDCLIENTE'];
                    $cddestfili = $parametros['CDDESTFILI'];

                    $this->distribuicaoRefeicaoService->atualizaQuantidadeProduto (
                        $nrorg, $cdfilial, $nrdistprod, $nrdistcard, $cdservfili, $idtipoprod, $dtcardapio,
                        $cdcliente, $cddestfili, $cdproduto, $qttotproduto
                    );
                }
			}
		} catch(\Exception $e) {
			$response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
		}
	}

	public function salvaDistribuicaoRefeicao(DTO\Request $request, DTO\Response $response){
		try {
			$parametros = $request->getRow();
			$nrorg = $parametros['NRORG'];
			$nrDistCard = $parametros['NRDISTCARD'];
			$nrDistProd = $parametros['NRDISTPROD'];
			$comensais = $parametros['QTCOMENSAIS'];
			$filial = $parametros['CDFILIAL'];
			$servico = $parametros['CDSERVFILI'];
			$cliente = $parametros['CDCLIENTE'];
			$destino = $parametros['CDDESTFILI'];
			$seqPrato = $parametros['NRSEQPRATCAR'];
			$embalagem = $parametros['CDCONTAINER'];
			$propEmb = $parametros['IDPROPOSITO'];
			$quantidade = $parametros['QTCONTAINER'];
			$agrupEmb = $parametros['CDAGRUPEMB'];
			$idPrincipal = $parametros['IDPRINCIPAL'];
			$observacao = $parametros['DSOBSETIQUETA'];
			$qtPrato = $parametros['QTPRATO'];
			$qtExtra = $parametros['QTEXTRA'];
			$tipoProd = $parametros['IDTIPOPROD'];
			
			$this->distribuicaoRefeicaoService->iniciaConexao();
			
			if ($tipoProd == 'NENHUM') {
				$this->trataPratoPrincipal($nrorg, $nrDistCard, $idPrincipal, null);
				$this->distribuicaoRefeicaoService->salvaDistribuicaoPrato(
					$nrorg, $nrDistCard, $nrDistProd, $cliente, $destino,
					$seqPrato, $quantidade, $agrupEmb,
					$observacao, $propEmb, $embalagem, $idPrincipal, $qtPrato,
					$qtExtra, $comensais
				);
			} else {
				$this->trataProdutoPrincipal($nrorg, $nrDistProd, $filial, $servico, $cliente, $destino, $idPrincipal, null, $nrDistCard);
				$this->distribuicaoRefeicaoService->salvaDistribuicaoProduto(
					$nrorg, $filial, $servico, $cliente, $destino, $nrDistProd, $quantidade, $agrupEmb, $observacao,
					$embalagem, $idPrincipal, $qtPrato, $comensais, $nrDistCard
				);
			}
			$this->distribuicaoRefeicaoService->realizaCommit();
		} catch(\Exception $e) {
			$this->distribuicaoRefeicaoService->realizaRollBack();
			$response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
		}
	}

	protected function trataPratoPrincipal($nrorg, $nrDistCard, $idPrincipal, $nrembdistcard){
		if ($idPrincipal === 'S')
			$valida = $this->distribuicaoRefeicaoService->verificaPratoPrincipal($nrorg, $nrDistCard, $nrembdistcard);
		else $valida = false;
		if ($valida) {
			$this->distribuicaoRefeicaoService->atualizaPratoPrincipal($valida, $nrorg);
		}
	}

	protected function trataProdutoPrincipal($nrorg, $nrDistProd, $filial, $servico, $cliente, $destino, $idPrincipal){
		if ($idPrincipal === 'S')
			$valida = $this->distribuicaoRefeicaoService->verificaProdutoPrincipal($nrorg, $nrDistProd, $filial, $servico, $cliente, $destino);
		else $valida = false;

		if ($valida) {
			foreach($valida as $itens) {
				$this->distribuicaoRefeicaoService->atualizaProdutoPrincipal(
					$itens, $nrorg
				);
			}
		}
	}

	public function excluiDistribuicaoRefeicao(DTO\Request $request, DTO\Response $response){
		try {
			$parametros = $request->getRow();
			$nrEmbDistCard = $parametros['NREMBDISTCARD'];
			$sequencia = $parametros['NRSEQPRATCAR'];
			$nrDistCard = $parametros['NRDISTCARD'];
			$nrorg = $parametros['NRORG'];
			$tipo = $parametros['IDTIPOPROD'];
			$nrDistProd = $parametros['NRDISTPROD'];

			if ($tipo === 'NENHUM') {
				$exclusao
					= $this->distribuicaoRefeicaoService->excluiDistribuicaoPrato(
					$nrorg, $nrEmbDistCard, $sequencia, $nrDistCard
				);
			} else {
				$exclusao
					= $this->distribuicaoRefeicaoService->excluiDistribuicaoProduto(
					$nrorg, $nrEmbDistCard, $nrDistProd, $nrDistCard
				);
			}

			$response->addDataSet(new DataSet ('EXCLUIDIST', array('P_RETORNO' => $exclusao)));
		} catch(\Exception $e) {
			$response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
		}
	}

	public function verificarEmbalagens(DTO\Request $request, DTO\Response $response){
		$parametros = $request->getRow();
		if ($this->verificaEmbdistprod($parametros['NRDISTPROD'], $parametros['NRORG']) || $this->verificaEmbdistcard($parametros['NRDISTPROD'], $parametros['NRORG'])){
			$response->addDataSet(new DataSet('coz07200_verificarembalagens', [true]));
		} else {
			$response->addDataSet(new DataSet('coz07200_verificarembalagens', [false]));
		}
	}

	private function verificaEmbdistprod($nrdistprod, $nrorg) {
		$conditions = array(
			array(
	       		'columnName' => 'NRDISTPROD',
	       		'operator' => '=',
	       		'value' => $nrdistprod
	      	),
	      	array(
	  			'columnName' => 'NRORG',
	  			'operator' => '=',
	  			'value' => $nrorg
	      	)
		);
		$dataSet = $this->crudService->getFilteredDataSet('embdistprod', $conditions);
		return (!empty($dataSet->getRows()));
	}

	private function verificaEmbdistcard($nrdistprod, $nrorg) {
		$conditions = array(
			array(
	       		'columnName' => 'NRDISTPROD',
	       		'operator' => '=',
	       		'value' => $nrdistprod
	      	),
	      	array(
	  			'columnName' => 'NRORG',
	  			'operator' => '=',
	  			'value' => $nrorg
	      	)
		);
		$dataSet = $this->crudService->getFilteredDataSet('coz07200_embdistcard_carddistprod', $conditions);
		return (!empty($dataSet->getRows()));
	}
	
	public function calcularEmbalagens(DTO\Request $request, DTO\Response $response){
		try {
			$parametros = $request->getRow();
			$nrorg = $parametros['NRORG'];
			$nrdistprod = $parametros['NRDISTPROD'];
			$retorno = $this->distribuicaoRefeicaoService->CalculaEmbalagens($nrdistprod, $nrorg);
            $response->addDataSet(new DataSet('RETORNO', array($retorno['P_RETORNO'])));
		} catch(\Exception $e) {
			$response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
		}
	}

    public function retornaParametroPorAgregada(DTO\Request\Row $request, DTO\Response $response){
        $parametros = $request->getRow();
        $nrdistprod = $parametros['NRDISTPROD'];
        $produtos = $parametros['PRODUTOS'];
        
        //Remove dados duplicados do array object
        $input = array_unique(array_column($produtos, 'CDPRODUTO'));
        $produtos = array_intersect_key($produtos, $input);
        
		$nrlista = $this->novoCodigo->getCodeByTableName('LISTA',$this->environment->getNrorgTrab());
        $sql = "INSERT INTO LISTA (NRLISTA, CDLISTA) VALUES (:NRLISTA, :CDLISTA)";
		foreach ($produtos as $produto) {
			$stmt = $this->entityManager->getConnection()->prepare($sql);
			$stmt->execute(
                    array(
                        "NRLISTA" => $nrlista,
                        "CDLISTA" => $produto['CDPRODUTO']
                    )
                );
		}
		
		$filterCriteria = new \Zeedhi\Framework\DataSource\FilterCriteria('paramplaneja_agregada_dist_prod');
        $this->parameterBagColumns = array('P_NRORG', 'P_NRDISTPROD', 'P_NRLISTA');

        $filterCriteria->addCondition('P_NRORG', $this->environment->getNrorgTrab());
        $filterCriteria->addCondition('P_NRDISTPROD', $nrdistprod);
        $filterCriteria->addCondition('P_NRLISTA', $nrlista);
        $filterCriteria->setDataSourceName('paramplaneja_agregada_dist_prod');
		
		$newFilterCriteria = $this->crudService->populateParameterBag($filterCriteria, $this->parameterBagColumns);
        $this->crudService->find($response, $newFilterCriteria);
        $this->util->deleteLista($nrlista);
    }
    
    public function geraListaProduto(DTO\Request\Row $request, DTO\Response $response) {
     try {          
          $data = $request->getRow();
          $listafiliserv = $data["FILISERV"];
          $nrlista = $this->novoCodigo->getCodeByTableName('LISTA',$data["NRORG"]);
          
          $sql = "INSERT INTO LISTA (NRLISTA, CDLISTA) VALUES (:NRLISTA, :CDLISTA)";
          if($listafiliserv[0] == "T"){
                $listafiliserv = $this->retornaFiliserv($data["NRORG"],$data["CDFILIAL"],$data["CDCENTRAL"]);
		  }
          foreach ($listafiliserv as $filiserv) {
        		$stmt = $this->entityManager->getConnection()->prepare($sql);
        		if(is_array($filiserv)){
        			$stmt->execute(
	                    array(
	                        "NRLISTA" => $nrlista,
	                        "CDLISTA" => $filiserv['FILISERV']
	                    )
	                );
        		}else{
        			$stmt->execute(
	                    array(
	                        "NRLISTA" => $nrlista,
	                        "CDLISTA" => $filiserv
	                    )
	                );
        		}
            }
          
            
            $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet ('nrLista', array($nrlista)));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), 1));
        }
    }
    
    public function retornaFiliserv($nrorg,$cdfiliais,$cdCentral){
        
        if($cdfiliais[0] == 'T'){
        	$concatCdfilial = $cdfiliais[0];
        }else{
	        $array = [];
        	foreach ($cdfiliais as $filial) {
        		array_push($array, $filial['CDFILIAL']);
        	}	
        	$concatCdfilial = join("_", $array);
        }
        
        $params = array(
            'NRORG'         => $nrorg,
            'CDFILIAGREGA'  => $concatCdfilial,
            'CDGRUPSERV'    => "T",
            'CDGRUPFILI'    => "T",
            'CDCENTRAL'     => $cdCentral
        );

        $sql = "SELECT DISTINCT S.CDFILIAL, FILIAL.NMFILIAL, S.CDSERVFILI, S.NMSERVICO, S.CDGRUPSERV, S.CDFILIAL ||S.CDSERVFILI AS FILISERV
                FROM SERVICO S INNER JOIN(SELECT FL.CDFILIAL FROM FILGRUPFILI FL INNER JOIN FILIAL FI ON ((FI.CDFILIAL = FL.CDFILIAL)
                AND (INSTR(:CDCENTRAL, FI.CDFILCENTPME) <> 0) AND FI.NRORG = :NRORG) 
                WHERE ((INSTR(:CDGRUPFILI, FL.CDGRUPFILI))<> 0 OR (:CDGRUPFILI = 'T')) AND ((INSTR(:CDFILIAGREGA, FL.CDFILIAL)) <> 0 OR (:CDFILIAGREGA = 'T')) 
                AND FL.NRORG = :NRORG UNION ALL SELECT CDFILIAL FROM FILIAL WHERE :CDGRUPFILI = 'T' AND (INSTR(:CDCENTRAL, CDFILCENTPME) <> 0) AND 
                ((INSTR(:CDFILIAGREGA, CDFILIAL))<> 0 OR (:CDFILIAGREGA = 'T')) AND NRORG = :NRORG UNION ALL SELECT CDFILIAGREGA AS CDFILIAL 
                FROM CELULAPROD WHERE (INSTR(:CDCENTRAL, CDCENTRAL) <> 0) AND ((INSTR(:CDFILIAGREGA, CDFILIAGREGA))<> 0 OR (:CDFILIAGREGA = 'T')) AND (NRORG = :NRORG)) FIL ON ((S.CDFILIAL = FIL.CDFILIAL) 
                AND ((INSTR(:CDGRUPSERV, S.CDGRUPSERV) <> 0) OR (:CDGRUPSERV = 'T'))) AND IDSERVTRAN = 'S' INNER JOIN FILIAL ON FILIAL.CDFILIAL = S.CDFILIAL AND FILIAL.NRORG = S.NRORG AND FILIAL.IDCNTRRESC = 'N'";
    
        $data = $this->entityManager->getConnection()->fetchAll($sql, $params);
        $result = [];
        
        foreach ($data as $row) {
            array_push($result,$row["FILISERV"]);
        }
        return $result;  
    }
    
	public function deletaListaProduto(DTO\Request\Row $request, DTO\Response $response) {
	    try {
	    	  
	          $data = $request->getRow();
	          $this->util->deleteLista($data["NRLISTA"]);
	        
	    } catch (\Exception $e) {
	        $response->setError(new DTO\Response\Error($e->getMessage(), 1));
	    }
	}
	
	public function saveDistribuicao(DTO\Request\Row $request, DTO\Response $response){
		try{
			$row = $request->getRow();
			
			$row['__is_new'] = true;
			$row['NRDISTPROD'] = intval($this->novoCodigo->getCodeByTableName('DISTPROD',$this->environment->getNrOrgTrab()));
			$row['IDEFETIVADA'] = "N";
			
			$newRequest = new DTO\Request\Row($row,"","","");
		    $dataSourceName = 'distprod';
		    $rowToSave = $this->crudService->getRowsToSave($newRequest, $dataSourceName);
		    $this->crudService->save($dataSourceName,$rowToSave);
		     
		    $response->addDataSet(new DataSet('NRDISTPROD',array($row['NRDISTPROD'])));
		}catch (\Exception $e) {
           $response->setError(new Error($e->getMessage(), 500));
        }
	}

	public function saveProdutoDistribuicao(DTO\Request\Row $request, DTO\Response $response){
		
		try{
            $this->connection->beginTransaction();

			$rows = $request->getRow()["rows"];
	        $params = $request->getRow()["params"];
			
        	foreach ($rows as $row) {
        		$nrSeqProd = $this->retornaNovoNrseqProd($row,$params,'proddistprod');
        		$this->savePADist($row,$params,$nrSeqProd);
        	}
           
            $this->connection->commit();
        }catch (\Exception $e) {
           $this->connection->rollBack();
           $response->setError(new Error($e->getMessage(), 500));
        }
		
	}
	
    public function deleteProduto(DTO\Request\Row $request, DTO\Response $response){
        try{
        	
            $rows =  $request->getRow();
            $dataSourceName = "proddistprod";
            $this->connection->beginTransaction();
            
            foreach ($rows as $row) {
            	 
            	$rowsProdDistProd = $this->retornaProdDistProd($row);
				foreach($rowsProdDistProd as $rowsProd){
					 $rowsItem = $this->deleteItemEmbDistProd($rowsProd);
					 $this->deleteEmbDistProd($rowsItem);
					 $newRequest = new DTO\Request\Row($rowsProd,"","","");
	                 $rowToDelete = $this->crudService->getRowsToDelete($newRequest,$dataSourceName);
	             	 $dataSet = new DataSet($dataSourceName, $rowToDelete);
	                 $this->dataSourceManager->delete($dataSet);
				}
	 
            }
            $this->connection->commit();
        }catch (\Exception $e) {
           $this->connection->rollBack(); 
           $response->setError(new Error($e->getMessage(), 500));
        }     
        
    }
    
    private function deleteItemEmbDistProd($rowsProd){
    	$dataSourceName = 'itembdistprod';
    	
    	$conditions = array(
    	array(
	    	"columnName" => "NRPRODDISTPROD",
			"operator" => "=",
			"value" => $rowsProd["NRPRODDISTPROD"]
    	    ),
    	array(
	    	"columnName" => "NRDISTPROD",
			"operator" => "=",
			"value" => $rowsProd["NRDISTPROD"]
    	    ),
    	array(
	    	"columnName" => "NRORG",
			"operator" => "=",
			"value" => $rowsProd["NRORG"]
    	    )
    	);
		$dataSet = $this->crudService->getFilteredDataSet($dataSourceName, $conditions);
        $rowsItem = $dataSet->getRows();

        foreach ($rowsItem as $row) {
	        $newRequest = new DTO\Request\Row($row,"","","");
	    	$rowToDelete = $this->crudService->getRowsToDelete($newRequest,$dataSourceName);
	 		$dataSet = new DataSet($dataSourceName, $rowToDelete);
	    	$this->dataSourceManager->delete($dataSet);
        }
        
        return $rowsItem;
    }
    
    private function deleteEmbDistProd($rowsItem){
    	$dataSourceName = 'embdistprod';
    	
        foreach ($rowsItem as $row) {
	        $newRequest = new DTO\Request\Row($row,"","","");
	    	$rowToDelete = $this->crudService->getRowsToDelete($newRequest,$dataSourceName);
	 		$dataSet = new DataSet($dataSourceName, $rowToDelete);
	    	$this->dataSourceManager->delete($dataSet);
        }
    }
    
    public function retornaProdDistProd($row){
    	$params = array(
            'CDPRODUTO'       =>  $row['CDPRODUTO'],
        	'IDORIGEM'		  =>  $row['IDORIGEM'],
        	'NRDISTPROD'	  =>  $row['NRDISTPROD']
        );
        $sql = "SELECT * FROM PRODDISTPROD WHERE CDPRODUTO =:CDPRODUTO AND IDORIGEM =:IDORIGEM AND NRDISTPROD =:NRDISTPROD";
        $rowProdDistProd= $this->entityManager->getConnection()->fetchAll($sql, $params);
        return $rowProdDistProd;
    }
    
    public function deleteUnidadeAgreg(DTO\Request\Row $request, DTO\Response $response){
        try{
            $rows =  $request->getRow();
            $dataSourceName = "proddistprod";
            $this->connection->beginTransaction();
            foreach ($rows as $row) {
               $row['NRPRODDISTPROD'] = $row['NRDISTCARD'];
               $newRequest = new DTO\Request\Row($row,"","","");
	           $rowToDelete = $this->crudService->getRowsToDelete($newRequest,$dataSourceName);
	           $dataSet = new DataSet($dataSourceName, $rowToDelete);
	           $this->dataSourceManager->delete($dataSet);      
	        }
            $this->connection->commit();
        }catch (\Exception $e) {
           $this->connection->rollBack(); 
           $response->setError(new Error($e->getMessage(), 500));
        }     
        
    }
	
	public function savePADist($row,$params,$nrSeqProd){
			 
			if(empty($params['QTPORPESSOA'])){
			 	$qntPorPessoa = $this->retornaQtPessoaProduto($row,$params);
			 }else{
			 	$qntPorPessoa = $params['QTPORPESSOA'];
			 }
			 
			 if ($qntPorPessoa <= 0 ) throw new \Exception("PROD_ACABADO_SEL_SEM_ASSOC");
			 
			 $row['__is_new']         = true;
			 $row['NRPRODDISTPROD']	  = $nrSeqProd;
             $row['NRDISTPROD']	      = $params['NRDISTPROD'];
             $row['DTCARDAPIO']       = $params['DTCONSUMO'];
             $row['CDPRODUTO']        = $params['CDPRODUTO'];
             $row['QTCOMENSAIS']      = $row['COMENSAIS'];
             $row['QTTOTCOMENSAIS']	  = $row['COMENSAIS'];
			 $row['QTPORPESSOA']	  = $qntPorPessoa;
             $row['QTTOTPRODUTO']	  = $qntPorPessoa * $row['COMENSAIS'];
			 $row['IDTIPOPROD']		  = "DISTRIBUICAO";
			 $row['IDORIGEM']		  = "MANUAL";
             $newRequest = new DTO\Request\Row($row,"","","");
             $dataSourceName = 'proddistprod';
             $rowToSave = $this->crudService->getRowsToSave($newRequest, $dataSourceName);
             $this->crudService->save($dataSourceName,$rowToSave);
	}
	
	private function retornaQtPessoaProduto($row,$params){
		$qntPorPessoa = 0;
		$sql = "SELECT QTPORPESSOA FROM(SELECT QTPADRAOSERV AS QTPORPESSOA, CDPRATO, CDFILAUXPRAT FROM PRATSERV WHERE CDFILIAL = :CDFILIAL AND CDSERVFILI = :CDSERVFILI AND CDPRODPRODUCAO = :CDPRODUTO AND QTPADRAOSERV IS NOT NULL UNION ALL SELECT QTPADRAOFILI AS QTPORPESSOA, CDPRATO, CDFILAUXPRAT 
				FROM PRATFILI WHERE CDFILIAL = :CDFILIAL AND CDPRODPRODUCAO = :CDPRODUTO AND QTPADRAOFILI IS NOT NULL AND NOT EXISTS (SELECT QTPADRAOSERV FROM PRATSERV WHERE CDFILIAL = :CDFILIAL AND CDSERVFILI = :CDSERVFILI AND CDPRODPRODUCAO = :CDPRODUTO AND QTPADRAOSERV IS NOT NULL) 
				UNION ALL SELECT QTPADRAOPRAT AS QTPORPESSOA, CDPRATO, CDFILAUXPRAT FROM PRATPADR WHERE CDPRODPRODUCAO = :CDPRODUTO AND QTPADRAOPRAT IS NOT NULL AND NOT EXISTS (SELECT QTPADRAOSERV FROM PRATSERV WHERE CDFILIAL = :CDFILIAL AND CDSERVFILI = :CDSERVFILI AND CDPRODPRODUCAO = :CDPRODUTO AND QTPADRAOSERV IS NOT NULL) 
				AND NOT EXISTS (SELECT QTPADRAOFILI AS QTPORPESSOA FROM PRATFILI WHERE CDFILIAL = :CDFILIAL AND CDPRODPRODUCAO = :CDPRODUTO AND QTPADRAOFILI IS NOT NULL) ) A ORDER BY CDFILAUXPRAT";
		$params = array(
            'CDSERVFILI'   => $row["CDSERVFILI"],
            'CDFILIAL'     => $row["CDFILIAL"],
            'CDPRODUTO'    => $params["CDPRODUTO"]
        );
        
        $data= $this->entityManager->getConnection()->fetchAll($sql, $params);
        if (count($data) > 0) $qntPorPessoa = $data[0]['QTPORPESSOA'];
        return $qntPorPessoa;
	}
	
	private function retornaNovoNrseqProd($row,$params,$table){
		 if(!is_string($params)) $nrdistprod = $params["NRDISTPROD"];
		 else $nrdistprod = $params;

		 $params = array(
            'NRDISTPROD'    => $nrdistprod
        );
        $sql = "SELECT MAX(NRPRODDISTPROD) AS MAXNRPRODDISTPROD FROM $table
                WHERE NRDISTPROD =:NRDISTPROD";
    
        $data= $this->entityManager->getConnection()->fetchAll($sql, $params);
        $nrSeqProd = $data[0]['MAXNRPRODDISTPROD'] + 1;
        return $nrSeqProd;
	}
	
	public function verificarDescricao(DTO\Request\Row $request, DTO\Response $response){
		$row = $request->getRow();
		$descricaoAtual = $row['DSDISTRIBREFEICAO'];
		$descricao[0] = false;

		$query = "SELECT DSDISTRIBREFEICAO FROM DISTPROD
							WHERE NRDISTPROD = :NRDISTPROD 
							AND CDCENTRAL = :CDCENTRAL";

        $params = array(
            'NRDISTPROD' => $row["NRDISTPROD"],
            'CDCENTRAL'  => $row["CDCENTRAL"]
        );
        
        $stmt = $this->connection->prepare($query);
        $stmt->execute($params);
        $result = $stmt->fetchAll(\PDO::FETCH_ASSOC);
        
        if($result[0]['DSDISTRIBREFEICAO'] != $descricaoAtual){
        	$descricao[0] = true;
        }
        $response->addDataSet(new DataSet("descricao", $descricao));
	}
	
	public function cancelarEfetivacao(DTO\Request\Row $request, DTO\Response $response){
		try {
			
			$row = $request->getRow();
			$verificacaoOK = true;
			$nrdistprod = $row['NRDISTPROD'];
			$arrayProdutos = $row['CDPRODSELECIONADOS'];
        	$nrlista = $this->gerarLista($arrayProdutos);
        	
        	/* Cancelamento por produto */
        	$cancelDist = $row['CANCELDIST'];
        	if(!$cancelDist){ 
        		$retorno = $this->distribuicaoRefeicaoService->verificarNota($nrlista, $nrdistprod);
        		if(!empty($retorno)){ // Não pode cancelar
					return $response->addDataSet(new DataSet ('EFETIVADIST', array('P_RETORNO' => 'INCONSISTENCIA_NOTA')));
        		}
        	}
        	
        	$row['NRLISTAPROD'] = $nrlista;
        	//unset($row['CDPRODSELECIONADOS']);
        	//unset($row['CANCELDIST']);
        	
        	$request = new DTO\Request\Row($row,'', '', '');
           	$this->procedureEfetuaDistribuicao($request, $response);
  
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        } 
	}
	
	public function gerarLista($arrayProdutos){
        try {
            $this->nrlista = $this->util->insertLista($arrayProdutos);
            return $this->nrlista;
            
        } catch (\Exception $e) {
            throw $e;
        } 
	}
	
	public function gerarListaProdutos(DTO\Request\Row $request, DTO\Response $response){
        try {
            $row = $request->getRow();
            $this->nrlista = $this->util->insertLista($row['CDPRODSELECIONADOS']);
            $nrLista = $this->nrlista;
            $response->addDataSet(new DataSet("lista", array ("numeroLista" => $nrLista)));
            
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        } 
	}
	
	public function verificarDistribuicao(DTO\Request\Row $request, DTO\Response $response){
		$row = $request->getRow();
		$nrdistcard = $row['NRDISTPROD'];

		$query = "SELECT IDEFETIVADA FROM DISTPROD WHERE NRDISTPROD = :NRDISTPROD";

        $params = array(
            'NRDISTPROD' => $row["NRDISTPROD"]
        );
        
        $stmt = $this->connection->prepare($query);
        $stmt->execute($params);
        $result = $stmt->fetchAll(\PDO::FETCH_ASSOC);
        $idefetivada = $result[0]['IDEFETIVADA'];
        
        $response->addDataSet(new DataSet("EFETIVADA", $result[0]));
	}
	
	public function verificaProdutosManuais(DTO\Request $request, DTO\Response $response){
		$row = $request->getRow();
		if(!empty($row["SERVICOS"])) {
			$servicos = explode("_", $row["SERVICOS"]);
			$nrlista = $this->util->insertLista($servicos);
			$queryLista = " INNER JOIN LISTA L ON L.NRLISTA =:NRLISTA AND L.CDLISTA = F.CDFILIAGREG || F.CDSERVFILIAGREG ";
		}else {
			$nrlista = null;
			$queryLista = " ";
		}
		
		$produtos = [];
		$sql = "SELECT DISTINCT P.CDPRATO, P.CDFILAUXPRAT, P.CDPRODACABADO AS CDPRODUTO, F.DTCONSUMO, F.CDFILIAGREG AS CDFILIAL, F.CDSERVFILIAGREG AS CDSERVFILI, 
				        F.QTCOMENSSERV, ROUND(F.QTAPRODUZIR,5) AS QTAPRODUZIR,( F.QTAPRODUZIR / F.QTCOMENSSERV) AS QTPADRAO, 'DISTRIBUICAO' AS IDTIPOPROD,
				        PR.CDTIPOPRAT
				FROM PLANPRODUCAO PP INNER JOIN PLANPRODACABADO P ON P.IDORIGEM = 'MANUAL' AND P.NRPLANPROD = PP.NRPLANPROD 
				INNER JOIN PLANPRODFILIAGREGA F ON F.NRPLANPROD = P.NRPLANPROD AND F.CDPRODACABADO = P.CDPRODACABADO AND F.IDCOOKCHILL = P.IDCOOKCHILL 
				AND F.IDFINALIZACAO = P.IDFINALIZACAO AND F.IDTPPRODCHILL = P.IDTPPRODCHILL AND F.IDESTADO = 'CONSUMO' AND F.IDESTADO = P.IDESTADO 
				AND F.NRSEQPRODACABADO = P.NRSEQPRODACABADO AND F.IDPRODUZIR = 'S' /* AND F.IDCONCLUIDO = 'S' */AND F.DTCONSUMO BETWEEN :DTCONSUMOIN AND :DTCONSUMOFIN
				INNER JOIN PLANRECPROD PR ON PR.NRPLANPROD = F.NRPLANPROD AND PR.CDPRODACABADO = F.CDPRODACABADO AND PR.IDCOOKCHILL = F.IDCOOKCHILL
				AND PR.IDFINALIZACAO = F.IDFINALIZACAO AND PR.IDTPPRODCHILL = F.IDTPPRODCHILL AND PR.IDESTADO = F.IDESTADO 
				AND PR.NRSEQPRODACABADO = F.NRSEQPRODACABADO AND PR.DTCONSUMO BETWEEN :DTCONSUMOIN AND :DTCONSUMOFIN
					".$queryLista."
				WHERE PP.CDFILIAL = :CDCENTRAL
				AND P.CDPRODACABADO NOT IN (
				    SELECT PDP.CDPRODUTO
				    FROM PRODDISTPROD PDP
				    WHERE PDP.DTCARDAPIO BETWEEN :DTCONSUMOIN AND :DTCONSUMOFIN
				    AND PDP.CDFILIAL = F.CDFILIAGREG AND PDP.CDSERVFILI = F.CDSERVFILIAGREG
				)
				ORDER BY P.CDPRATO, P.CDPRODACABADO, F.CDFILIAGREG, F.CDSERVFILIAGREG, F.DTCONSUMO";
				
		$params = array(
	        'DTCONSUMOIN'   => $row["DTCONSUMOIN"],
	        'DTCONSUMOFIN'  => $row["DTCONSUMOFIN"],
	        'CDCENTRAL' 	=> $row["CDCENTRAL"],
	        'NRLISTA' 		=> $nrlista
	    );
	    
	    $data= $this->entityManager->getConnection()->fetchAll($sql, $params);
		if(!empty($row["SERVICOS"])) $this->util->deleteLista($nrlista);
		$response->addDataSet(new DataSet('produtosManuais', $data));
	}
	
	public function salvaProdManuaisDist(DTO\Request $request, DTO\Response $response){
		$row = $request->getRow();
		try{
			$this->connection->beginTransaction();
			
			$produtos = $row[0];
			$nrdistprod = $row[1];
			$filter = $row[2];
            
            
            if($nrdistprod == '0' || empty($nrdistprod)) $nrdistprod = $this->criaDistribuicao($filter);
            
			foreach($produtos as $produto){
					$nrSeqProd = $this->retornaNovoNrseqProd($row, $nrdistprod,'proddistprod');
					 $produto['__is_new']         = true;
					 $produto['NRPRODDISTPROD']	  = $nrSeqProd;
		             $produto['NRDISTPROD']	      = $nrdistprod;
		             $produto['DTCARDAPIO']       = $produto['DTCONSUMO'];
		             $produto['QTCOMENSAIS']      = $produto['QTCOMENSSERV'];
		             $produto['QTTOTCOMENSAIS']	  = $produto['QTCOMENSSERV'];
					 $produto['QTPORPESSOA']	  = $produto['QTPADRAO'];
					 $produto['QTTOTPRODUTO']	  = $produto['QTAPRODUZIR'];
					 $produto['CDTIPOPRAT']		  = $produto['CDTIPOPRAT'];
					 //$produto['CDPRODUTO']		  = $produto['CDPRODACABADO'];
					 $produto['IDORIGEM']		  = "MANUAL";
		             $newRequest = new DTO\Request\Row($produto,"","","");
		             
		             $dataSourceName = 'proddistprod';
		             $rowToSave = $this->crudService->getRowsToSave($newRequest, $dataSourceName);
		             $this->crudService->save($dataSourceName,$rowToSave);
					 $this->salvaReceita($produto, $nrdistprod, $row);
			}
			
			
			$this->connection->commit();
			$response->addDataSet(new DataSet('nrdistprod', [$nrdistprod]));
		}catch (\Exception $e) {
           $this->connection->rollBack();
           $response->addDataSet(new DataSet('nrdistprod', ["error"]));
        }
	}
	
	private function criaDistribuicao($filter) {
		$nrorg = $this->environment->getNrorgTrab();
		$data = $this->environment->getCurrentTime()->format("d/m/Y H:i:s");
		$newNrdistprod = $this->novoCodigo->getCodeByTableName('DISTPROD', $nrorg);
		
		$query = "INSERT INTO DISTPROD (NRDISTPROD, CDCENTRAL, DTINIPROD, DTFINPROD, IDEFETIVADA, NRORG, IDATIVO, IDGERAREQUI,
				                      NRORGINCLUSAO, DTINCLUSAO, DTULTATU, CDOPERINCLUSAO)
				  VALUES (:NRDISTPROD, :CDCENTRAL, :DTINIPROD, :DTFINPROD, :IDEFETIVADA, :NRORG, :IDATIVO, :IDGERAREQUI,
				                      :NRORGINCLUSAO, :DTINCLUSAO, :DTULTATU, :CDOPERINCLUSAO)";

        $params = array(
            'NRDISTPROD' => $newNrdistprod,
            'CDCENTRAL' => $filter['CDCENTRAL'],
            'DTINIPROD' => $filter['DTCONSUMOIN'],
            'DTFINPROD' => $filter['DTCONSUMOFIN'],
            'IDEFETIVADA' => 'N',
            'NRORG' => $nrorg,
            'IDATIVO' => 'S',
            'IDGERAREQUI' => 'N',
            'NRORGINCLUSAO' => $nrorg,
            'DTINCLUSAO' => $data,
            'DTULTATU' => $data,
            'CDOPERINCLUSAO' => $this->environment->getCdOperador()
        );
        
        $stmt = $this->connection->prepare($query);
        $stmt->execute($params);
        return $newNrdistprod;
	}
	
	private function salvaReceita ($produto, $nrdistprod, $row){
		
		$sql = "SELECT R.CDPRODUTO, DECODE(R.IDFINALIZACAO,'M', ROUND(R.QTPRODUTO + R.QTCOMPLPROD,5), ROUND(R.QTPRODUTO,5)) AS QTTOTPRODUTO, R.QTPADRAO, DECODE(R.IDTPREQPROD,'E','ENTREGADIRETA', 'D', 'DISTRIBUICAO') AS IDTIPOPROD 
				FROM TABLE( RECEITA_UTIL(:CDFILIAL,:CDSERVFILI,:CDPRATO,:CDFILAUXPRAT,'0','Z',:CDPRATO,0,0,'S') ) R WHERE R.CDPRODPRODUCAO =:CDPRODACABADO AND NVL(R.IDPRODCHILL,'N') = 'N' AND NVL(R.QTPADRAO,0) > 0 AND NVL(R.IDTPREQPROD,'P') IN ( 'D', 'E' )";
		$params = array(
            'CDFILIAL'  	=> $produto["CDFILIAL"],
            'CDSERVFILI'	=> $produto["CDSERVFILI"],
            'CDPRATO' 		=> $produto["CDPRATO"],
            'CDPRODACABADO' => $produto['CDPRODUTO'],
            'CDFILAUXPRAT'	=> $produto['CDFILAUXPRAT']
        );
        
        $data = $this->entityManager->getConnection()->fetchAll($sql, $params);
		foreach ($data as $receita) {
			$nrSeqProd = $this->retornaNovoNrseqProd($row, $nrdistprod,'proddistprod');
			 $receita['__is_new']         = true;
			 $receita['NRPRODDISTPROD']	  = $nrSeqProd;
             $receita['NRDISTPROD']	      = $nrdistprod;
			 $receita['CDFILIAL']		  = $produto["CDFILIAL"];
			 $receita['CDSERVFILI']		  = $produto["CDSERVFILI"];
             $receita['DTCARDAPIO']       = $produto['DTCARDAPIO'];
             $receita['QTCOMENSAIS']      = $produto['QTCOMENSAIS'];
             $receita['QTTOTCOMENSAIS']	  = $produto['QTTOTCOMENSAIS'];
			 $receita['QTPORPESSOA']	  = $receita['QTPADRAO'];
			 $receita['QTTOTPRODUTO']     = $receita['QTTOTPRODUTO'] * $produto['QTCOMENSAIS'];
			 $receita['IDORIGEM']		  = "MANUAL";
             $newRequest = new DTO\Request\Row($receita,"","","");
             
             $dataSourceName = 'proddistprod';
             $rowToSave = $this->crudService->getRowsToSave($newRequest, $dataSourceName);
             $this->crudService->save($dataSourceName,$rowToSave);
		}
		
	}
	
}